From 32fc5ec2fa9c5431e1b5718ff8ab36467fb0e1cb Mon Sep 17 00:00:00 2001
From: Aditya Vidyadhar Kamath <Aditya.Kamath1@ibm.com>
Date: Tue, 8 Nov 2022 05:27:44 -0600
Subject: [PATCH] Fix Multi thread debug bug fix in AIX

In the recent commit 98ed24fb35d89eb20179edf6c12f599c7a9e228e made by Mr. Tom there is a change in aix-thread.c file that changes

static_cast <aix_thread_info *> in gdb to gdb::checked_static_cast <aix_thread_info *>

AIX folks using the latest version will not be able to debug multi thread programs as a result of it

The error in AIX is as follows:-

internal-error: checked_static_cast: Assertion 'result != nullptr' failed.

The reason is that once the threads are syncronised with sync_threadlists () and threads are added with priv -

We iterate over threads to get the thread who caused the event and return its ptid

However the pthreadlib library treats the main process and its thread as separate threads though they are one thread.

This patch is a fix for the same.
---
 gdb/aix-thread.c | 13 +++++++++++++
 1 file changed, 13 insertions(+)

diff --git a/gdb/aix-thread.c b/gdb/aix-thread.c
index e556c153576..07190543973 100644
--- a/gdb/aix-thread.c
+++ b/gdb/aix-thread.c
@@ -816,6 +816,19 @@ sync_threadlists (int pid)
 					 ptid_t (pid, 0, pbuf[pi].pthid),
 					 priv);
 
+	  /* When the new thread is added and the pthread library is 
+             initialised, the process is threaded but in the 
+             libpthread library it will be counted as two threads
+             one with the main process and second one with the thread
+             that is added.  The main process thread needs to have a
+             private data.  The thread we added will have but main 
+             process will not. Hence the below chunk code does this.  */
+
+          inferior *inf = find_inferior_pid (proc_target, pid);
+          for (thread_info *tp : inf->threads ())
+            if (tp->priv == NULL)
+              tp->priv.reset (priv); 
+
 	  pi++;
 	}
       else
-- 
2.31.1

